@@ -6,7 +6,7 @@ from __future__ import (unicode_literals, division, absolute_import,
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os
+import os, posixpath
 from binascii import hexlify
 from collections import OrderedDict, defaultdict, Counter
 from functools import partial
@@ -15,7 +15,7 @@ import sip
 from PyQt5.Qt import (
     QWidget, QTreeWidget, QGridLayout, QSize, Qt, QTreeWidgetItem, QIcon, QFont,
     QStyledItemDelegate, QStyle, QPixmap, QPainter, pyqtSignal, QMenu, QTimer,
-    QDialogButtonBox, QDialog, QLabel, QLineEdit, QVBoxLayout, QScrollArea,
+    QDialogButtonBox, QDialog, QLabel, QLineEdit, QVBoxLayout, QScrollArea, QInputDialog,
     QRadioButton, QFormLayout, QSpinBox, QListWidget, QListWidgetItem, QCheckBox)
 
 from calibre import human_readable, sanitize_file_name_unicode, plugins
@@ -210,6 +210,13 @@ class FileList(QTreeWidget):
             }.iteritems()}
         self.itemActivated.connect(self.item_double_clicked)
 
+    @property
+    def current_name(self):
+        ci = self.currentItem()
+        if ci is not None:
+            return unicode(ci.data(0, NAME_ROLE) or '')
+        return ''
+
     def get_state(self):
         s = {'pos':self.verticalScrollBar().value()}
         s['expanded'] = {c for c, item in self.categories.iteritems() if item.isExpanded()}
@@ -233,13 +240,25 @@ class FileList(QTreeWidget):
                 if q == name:
                     return c
 
-    def select_name(self, name):
+    def select_name(self, name, set_as_current_index=False):
         for parent in self.categories.itervalues():
             for c in (parent.child(i) for i in xrange(parent.childCount())):
                 q = unicode(c.data(0, NAME_ROLE) or '')
                 c.setSelected(q == name)
                 if q == name:
                     self.scrollToItem(c)
+                    if set_as_current_index:
+                        self.setCurrentItem(c)
+
+    def select_names(self, names, current_name=None):
+        for parent in self.categories.itervalues():
+            for c in (parent.child(i) for i in xrange(parent.childCount())):
+                q = unicode(c.data(0, NAME_ROLE) or '')
+                c.setSelected(q in names)
+                if q == current_name:
+                    self.scrollToItem(c)
+                    s = self.selectionModel()
+                    s.setCurrentIndex(self.indexFromItem(c), s.NoUpdate)
 
     def mark_name_as_current(self, name):
         current = self.item_from_name(name)
@@ -459,8 +478,10 @@ class FileList(QTreeWidget):
         if num > 0:
             m.addSeparator()
             if num > 1:
-                m.addAction(QIcon(I('modified.png')), _('&Bulk rename selected files'), self.request_bulk_rename)
-            m.addAction(QIcon(I('trash.png')), _('&Delete the %d selected file(s)') % num, self.request_delete)
+                m.addAction(QIcon(I('modified.png')), _('&Bulk rename the selected files'), self.request_bulk_rename)
+            m.addAction(QIcon(I('modified.png')), _('Change the file extension for the selected files'), self.request_change_ext)
+            m.addAction(QIcon(I('trash.png')), ngettext(
+                '&Delete the selected file', '&Delete the {} selected files', num).format(num), self.request_delete)
             m.addSeparator()
 
         selected_map = defaultdict(list)
@@ -524,7 +545,7 @@ class FileList(QTreeWidget):
         else:
             return QTreeWidget.keyPressEvent(self, ev)
 
-    def request_bulk_rename(self):
+    def request_rename_common(self):
         if not current_container().SUPPORTS_FILENAMES:
             error_dialog(self, _('Cannot rename'), _(
                 '%s books do not support file renaming as they do not use file names'
@@ -534,19 +555,38 @@ class FileList(QTreeWidget):
         names = {unicode(item.data(0, NAME_ROLE) or '') for item in self.selectedItems()}
         bad = names & current_container().names_that_must_not_be_changed
         if bad:
-            return error_dialog(self, _('Cannot rename'),
+            error_dialog(self, _('Cannot rename'),
                          _('The file(s) %s cannot be renamed.') % ('<b>%s</b>' % ', '.join(bad)), show=True)
+            return
         names = sorted(names, key=self.index_of_name)
-        categories = Counter(unicode(item.data(0, CATEGORY_ROLE) or '') for item in self.selectedItems())
-        fmt, num = get_bulk_rename_settings(self, len(names), category=categories.most_common(1)[0][0])
-        if fmt is not None:
-            def change_name(name, num):
-                parts = name.split('/')
-                base, ext = parts[-1].rpartition('.')[0::2]
-                parts[-1] = (fmt % num) + '.' + ext
-                return '/'.join(parts)
-            name_map = {n:change_name(n, num + i) for i, n in enumerate(names)}
-            self.bulk_rename_requested.emit(name_map)
+        return names
+
+    def request_bulk_rename(self):
+        names = self.request_rename_common()
+        if names is not None:
+            categories = Counter(unicode(item.data(0, CATEGORY_ROLE) or '') for item in self.selectedItems())
+            fmt, num = get_bulk_rename_settings(self, len(names), category=categories.most_common(1)[0][0])
+            if fmt is not None:
+                def change_name(name, num):
+                    parts = name.split('/')
+                    base, ext = parts[-1].rpartition('.')[0::2]
+                    parts[-1] = (fmt % num) + '.' + ext
+                    return '/'.join(parts)
+                name_map = {n:change_name(n, num + i) for i, n in enumerate(names)}
+                self.bulk_rename_requested.emit(name_map)
+
+    def request_change_ext(self):
+        names = self.request_rename_common()
+        if names is not None:
+            text, ok = QInputDialog.getText(self, _('Rename files'), _('New file extension:'))
+            if ok and text:
+                ext = text.lstrip('.')
+
+                def change_name(name):
+                    base = posixpath.splitext(name)[0]
+                    return base + '.' + ext
+                name_map = {n:change_name(n) for n in names}
+                self.bulk_rename_requested.emit(name_map)
 
     @property
     def selected_names(self):
@@ -829,6 +869,7 @@ class MergeDialog(QDialog):  # {{{
 
     def __init__(self, names, parent=None):
         QDialog.__init__(self, parent)
+        self.names = names
         self.setWindowTitle(_('Choose master file'))
         self.l = l = QVBoxLayout()
         self.setLayout(l)
@@ -854,9 +895,9 @@ class MergeDialog(QDialog):  # {{{
 
     @property
     def ans(self):
-        for b in self.buttons:
+        for n, b in zip(self.names, self.buttons):
             if b.isChecked():
-                return unicode(b.text())
+                return n
 
 # }}}
 
@@ -870,8 +911,9 @@ class FileListWidget(QWidget):
         self.layout().addWidget(self.file_list)
         self.layout().setContentsMargins(0, 0, 0, 0)
         self.forwarded_signals = {k for k, o in vars(self.file_list.__class__).iteritems() if isinstance(o, pyqtSignal) and '_' in k and not hasattr(self, k)}
-        for x in ('delete_done', 'select_name', 'request_edit', 'mark_name_as_current', 'clear_currently_edited_name'):
+        for x in ('delete_done', 'select_name', 'select_names', 'request_edit', 'mark_name_as_current', 'clear_currently_edited_name'):
             setattr(self, x, getattr(self.file_list, x))
+        self.setFocusProxy(self.file_list)
 
     def build(self, container, preserve_state=True):
         self.file_list.build(container, preserve_state=preserve_state)
@@ -880,6 +922,10 @@ class FileListWidget(QWidget):
     def searchable_names(self):
         return self.file_list.searchable_names
 
+    @property
+    def current_name(self):
+        return self.file_list.current_name
+
     def __getattr__(self, name):
         if name in self.forwarded_signals:
             return getattr(self.file_list, name)
